﻿/////////////////////////////////////////////////////////////////////////
//
// This module contains code to do a basic green screen.
//
// Copyright © Microsoft Corporation.  All rights reserved.  
// This code is licensed under the terms of the 
// Microsoft Kinect for Windows SDK (Beta) from Microsoft Research 
// License Agreement: http://research.microsoft.com/KinectSDK-ToU
//
/////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Coding4Fun.Kinect.Wpf;
using Microsoft.Research.Kinect.Nui; 

namespace WorkingWithDepthData
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        //Kinect Runtime
        Runtime nui = new Runtime(); 

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {         
            //UseDepthAndPlayerIndex e UseSkeletalTracking
            nui.Initialize(RuntimeOptions.UseDepthAndPlayerIndex | RuntimeOptions.UseSkeletalTracking);

            //registra o evento
            nui.DepthFrameReady += new EventHandler<ImageFrameReadyEventArgs>(nui_DepthFrameReady);

            //DepthAndPlayerIndex ImageType
            // Neste caso, o Kinect retorna também a informação do ser humano
            nui.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, 
                ImageType.DepthAndPlayerIndex); 

        }

        void nui_DepthFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            // Converte a informação obtida em informação de cor
            byte[] ColoredBytes = GenerateColoredBytes(e.ImageFrame); 

            // Cria uma imagem baseada nas cores retornadas
            PlanarImage image = e.ImageFrame.Image;
            image1.Source = BitmapSource.Create(image.Width, image.Height, 96, 96, PixelFormats.Bgr32, null, 
                ColoredBytes, image.Width * PixelFormats.Bgr32.BitsPerPixel/ 8); 



        }

        private byte[] GenerateColoredBytes(ImageFrame imageFrame)
        {
            
            int height = imageFrame.Image.Height;
            int width = imageFrame.Image.Width;

            // Informação de profundidade para cada pixel
            Byte[] depthData = imageFrame.Image.Bits; 


            // Estrutura para armazenar as cores de cada pixel
            //Height x Width x 4 (Red, Green, Blue, empty byte)
            Byte[] colorFrame = new byte[imageFrame.Image.Height * imageFrame.Image.Width * 4];

            //Bgr32  - Blue, Green, Red, empty byte
            //Bgra32 - Blue, Green, Red, transparency 
            
            // O último byte é a transparência
            const int BlueIndex = 0;
            const int GreenIndex = 1;
            const int RedIndex = 2;
                       

            var depthIndex = 0;
            // Percorre a informação de profundidade
            for (var y = 0; y < height; y++)
            {
                
                var heightOffset = y * width;

                for (var x = 0; x < width; x++)
                {

                    var index = ((width - x - 1) + heightOffset) * 4;

                    //var distance = GetDistance(depthData[depthIndex], depthData[depthIndex + 1]);
                    var distance = GetDistanceWithPlayerIndex(depthData[depthIndex], depthData[depthIndex + 1]);

                    if (distance <= 900)
                    {
                        //coisas próximas ficam azuis
                        colorFrame[index + BlueIndex] = 255;
                        colorFrame[index + GreenIndex] = 0;
                        colorFrame[index + RedIndex] = 0;
       
                    }
                    else if (distance > 900 && distance < 2000)
                    {
                        //coisas intermediárias ficam verdes
                        colorFrame[index + BlueIndex] = 0;
                        colorFrame[index + GreenIndex] = 255;
                        colorFrame[index + RedIndex] = 0;
                    }
                    else if (distance > 2000)
                    {
                        // Distância maior fica vermelho
                        colorFrame[index + BlueIndex] = 0;
                        colorFrame[index + GreenIndex] = 0;
                        colorFrame[index + RedIndex] = 255;
                    }


                    ////equal coloring for monochromatic histogram
                    //var intensity = CalculateIntensityFromDepth(distance);
                    //colorFrame[index + BlueIndex] = intensity;
                    //colorFrame[index + GreenIndex] = intensity;
                    //colorFrame[index + RedIndex] = intensity;

                    // Se for um jogador, exibe em amarelo
                    if (GetPlayerIndex(depthData[depthIndex]) > 0)
                    {
                        //we are the farthest
                        colorFrame[index + BlueIndex] = 0;
                        colorFrame[index + GreenIndex] = 255;
                        colorFrame[index + RedIndex] = 255;
                    }

                    //jump two bytes at a time
                    depthIndex += 2;
                }
            }

            return colorFrame;
        }

        private static int GetPlayerIndex(byte firstFrame)
        {
            //returns 0 = no player, 1 = 1st player, 2 = 2nd player...
            //bitwise & on firstFrame
            return (int)firstFrame & 7; 
        }


        private int GetDistanceWithPlayerIndex(byte firstFrame, byte secondFrame)
        {
            //offset by 3 in first byte to get value after player index 
            int distance = (int)(firstFrame >> 3 | secondFrame << 5);
            return distance;
        }

        const float MaxDepthDistance = 4000; // max value returned
        const float MinDepthDistance = 850; // min value returned
        const float MaxDepthDistanceOffset = MaxDepthDistance - MinDepthDistance;

        public static byte CalculateIntensityFromDepth(int distance)
        {
            //formula for calculating monochrome intensity for histogram
            return (byte)(255 - (255 * Math.Max(distance - MinDepthDistance, 0) 
                / (MaxDepthDistanceOffset)));
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            //cleanup
            nui.Uninitialize(); 
        }

        }

    }

